#ifndef EXAMPLE_THREADING
#define EXAMPLE_THREADING

/*{ TemplateName::.h ************************************************
 
 Empty template for new header files.
 Copy and adapt to your needs.
 
}*/
//{ Defines/ TypeDefs ****************************************************

#define LED_STAT1 GPIOC, 6
#define LED_STAT2 GPIOC, 7

//} Defines
//{ Includes *************************************************************

 // Basic set of helper functions
#include "stm32_io.h"
#include <stdlib.h>
#define USE_STDPERIPH_DRIVER
#include "stm32f10x.h"
#include "core_cm3.h"
#include "stm32f10x_conf.h"
#include "bits.h"
#include <string.h>
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "semphr.h"
#include "mpu_wrappers.h""


#ifdef EXTENSION_example_leds
#include "example_leds.h"
#endif

#ifdef EXTENSION_lcd_320x240_olimex
#include "lcd-320x240.h"
#endif
//} Includes
//{ Structures/ Enums ****************************************************

// continuation of stm32_io.h:ErrorCode_E
typedef enum {
    ec_et_InvalidQueueGiven
} ErrorCode_E;

typedef struct {
    // Column == -1 | Row == -1: continue at last position
    s8_t Column;   // >= 0: Column to move cursor to
    s8_t Row;      // >= 0: Row to move cursor to
    char Text[10]; // zero terminated string to print
    s16_t Value;   // passed as value to printf
} et_PrintJob_t;

/** data being sent to taskProducer()/ taskConsumer() */
typedef struct {
    xSemaphoreHandle Lock_Init;                   // used to lock this structure during task initialisation
    xSemaphoreHandle SemaphoreProducer2Consumer;  // semaphore to consumer tasks
        xQueueHandle QueuePrint;                  // queue to print on display
                 int DisplayRow;                  // row in display used for debug output
} ets_ThreadInfo_t;

//} Structures/ Enums
//{ Global Variables *****************************************************



//} Global Variables
//{ Function prototypes **************************************************

// basic initialization (called from threading_start() )
void threading_init();

// run demo
void threading_start();

/** graphical display of current queue size
 * @param Semaphore   Semaphore/ Queue which size should be displayed
 * @param QueuePrint  queue to print to
 */
void displayQueue(xSemaphoreHandle Semaphore, xQueueHandle QueuePrint);

// produces tasks to do
void taskProducer(void *pvParameters);

// consumes + executes tasks to do
void taskConsumer(void *pvParameters);

// adds given data as new print job to given queue
// Will block if queue is full
void queuePrintJob(xQueueHandle PrintQueue, s8_t Column, s8_t Row, const char* Text);
void queuePrintJobV(xQueueHandle PrintQueue, s8_t Column, s8_t Row, const char* Text, s16_t Value);

// central print task (display cannot be used from multiple tasks
void taskPrint(void *pvParameters);


//} Function prototypes

#endif //EXAMPLE_THREADING
